#include "SellersRec.h"

SellersRec::SellersRec(const char* pattern, size_t m, int error): ApproximateMatcher(false) {

	this-> m = m;
	this-> pattern = pattern;
	this-> error = error;
	n = 0;
	numberOfMatchings = 0;

	colCount = MAX(error+m, 2);

	columns = new int*[colCount];

	for(int i = 0; i < colCount; ++i) {
		columns[i] = new int[m+1];
		columns[i][0] = 0;
	}

	resetOnLineBreak();

	lineMaxSize = 	1 + //caracter fim de cadeia (ASCII 0)
					(m+error)*(1+5+4) + //10 posições para cada caracter (possivelmente) colorido
					10; //caracteres extras apenas para situar o casamento no texto

	textLine = new char[lineMaxSize];
	patLine = new char[lineMaxSize];

	charsFromPreviousWindow = 0;
		
}

void SellersRec::resetOnLineBreak() {
	//Caso base da programação dinâmica
	for(int i = 0; i <= m; ++i)
		columns[0][i] = i;

	currentColumn = 1;
	previousCol = 0;
}

int SellersRec::update(char* chunk, size_t chunkSize){
	int initialPosition = (n==0) ? 0 : 10 + m + error - 1;
	
	for(int i = initialPosition; i < chunkSize; ++i) {
		if(chunk[i] != '\n') {
			for(int j = 1; j <= m; ++j){
				int phi = (pattern[j-1] == chunk[i]) ? 0 : 1;

				columns[currentColumn][j] = 
				MIN(
					1 + MIN(columns[previousCol][j], columns[currentColumn][j-1]),
					phi + columns[previousCol][j-1]);
			}

			if(columns[currentColumn][m] <= error) {
				reconstruct(chunk, i);
				++numberOfMatchings;
			}

			//currentColumn+1 mod colCount
			++currentColumn;
			if(currentColumn == colCount)
				currentColumn = 0;

			//previousCol + 1 mod colCount
			++previousCol;
			if(previousCol == colCount)
				previousCol = 0;
		} else {
			//recomeça casamento no final da linha
			resetOnLineBreak();
		}

		++n;
	}

	return charsFromPreviousWindow = 10 + m+error-1;
}

ull SellersRec::getNumberOfMatchings() {
	return numberOfMatchings;
}

void SellersRec::reconstruct(char* chunk, int pos) {
	int i = pos, row = m, col = currentColumn, prevCol = previousCol;

	char* up = textLine + (lineMaxSize-1), *down = patLine + (lineMaxSize-1);
	*up = *down = 0;
	--up, --down;

	while(row > 0) {
		if(columns[col][row] == (columns[col][row-1] + 1)) {//remoção
			//_ (text)
			//ch
			*up = '_';
			--up;

			writeColoredChar(down, pattern[row-1], RED_COLOR);

			--row;
		} else if(columns[col][row] == (columns[prevCol][row] + 1)) {//inserção
			//ch (text)
			//_

			writeColoredChar(up, chunk[i], RED_COLOR);

			*down = '_';
			--down;

			--i;
			col = prevCol;
			--prevCol;
			if(prevCol < 0) prevCol = colCount - 1;
		} else { //casamento ou substituição
			int phi = (pattern[row-1] == chunk[i]) ? 0 : 1;

			//ch (text)
			//ch (pat)
			if(phi) {
				writeColoredChar(up, chunk[i], YELLOW_COLOR);
				writeColoredChar(down, pattern[row-1], YELLOW_COLOR);
			} else {
				*(up--) = chunk[i];
				*(down--) = pattern[row-1];
			}

			--i;
			--row;
			col = prevCol;
			--prevCol;
			if(prevCol < 0) prevCol = colCount - 1;
		}
	}

	//imprime 10 char's extras do texto
	for(int k = 0; k < 10 && i >= 0 && chunk[i] != '\n'; ++k, --i){
		*(down--) = ' ';
		*(up--) = chunk[i];
	}

	printf("%s\n%s\n\n", up+1, down+1);
}

/*
 * Escreve color + ch + RESET_COLOR. Mas o buffer é escrito de trás para frente
 */
void SellersRec::writeColoredChar(char*& buf, char ch, const char* color){
	if(COLORED_PRINTING_IS_ENABLED) {
		for(int i = strlen(RESET_COLOR) - 1; i >= 0; --i)
			*(buf--) = RESET_COLOR[i];
	}
	
	*(buf--) = ch;

	if(COLORED_PRINTING_IS_ENABLED) {
		for(int i = strlen(color) - 1; i >= 0; --i)
			*(buf--) = color[i];
	}
}
