#ifndef SELLERS_REC
#define SELLERS_REC

#include "ApproximateMatcher.h"

class SellersRec : public ApproximateMatcher {
public:
	SellersRec(const char* pattern, size_t m, int error);

	//Ver Matcher::update
	int update(char* chunk, size_t chunkSize);

	ull getNumberOfMatchings();
	
	const char* getAlgoName() {
		return "Sellers(Rec)";
	}

	void resetForNewFile() {
		n = 0; charsFromPreviousWindow = 0; numberOfMatchings = 0;
		resetOnLineBreak();
	}

private:

	const char* pattern; //o padrão
	size_t m; //o tamanho do padrão
	size_t n; //o tamanho (atual) do texto
	int error; //o erro máximo permitido
	
	//as últimas colunas da matriz de PD (precisamos de m+error colunas)
	//Funciona como uma lista circular.
	int** columns; 
	//a quantidade (máxima) de colunas da matriz de PD que guardaremos para viabilizar a reconstrução do alinhamento
	int colCount; 
	//o indice da coluna que será preenchida com o próximo caracter do texto
	int currentColumn;
	//o indice da última coluna preenchida.
	int previousCol;

	//A quantidade de caracteres último chunk de texto que são repetidos no chunk atual
	int charsFromPreviousWindow;

	ull numberOfMatchings;

	//Buffers para reconstruir o alinhamento
	char *textLine, *patLine;
	//O tamanho máximo desses buffers.
	//Cada um poderá conter (m+error) caracteres, além de 10 caracteres extras
	//	Cada um dos (n+error) caracteres poderão ser coloridos e, para isso, 
	//	precisamos de 10  posições para cada um deles
	size_t lineMaxSize;

	//Atualiza o ponteiro 'buf' escrevendo o caracter 'ch' com a cor 'color'
	void writeColoredChar(char*& buf, char ch, const char* color);

	//Zera o estado do algoritmo para evitar que o caracter '\n' seja tratado como parte do texto
	void resetOnLineBreak();

	//Imprime o alinhamento que termina na posição 'pos' do 'chunk' atual
	void reconstruct(char* chunk, int pos);
};

#endif