#ifndef UKKONEN
#define UKKONEN

#include "ApproximateMatcher.h"
#include "UkkonenAutomatonNode.h"
#include "UkkonenTreeNode.h"

#include <algorithm>
#include <queue>
#include <vector>
using std::vector;
using std::queue;
using std::pair;
using std::make_pair;

class Ukkonen : public ApproximateMatcher {
public:
	Ukkonen(bool countOnly, const char* pattern, size_t m, int error);
	
	//Ver Matcher::update
	int update(char* chunk, size_t chunkSize);
	
	const char* getAlgoName(){
		return "Ukkonen";
	}

	void resetForNewFile() {
		n = 0;
		currentAutomatonState = initialAutomatonState;
		numberOfMatchings = 0;
	}
private:
	vector<UkkonenTreeNode> tree; //Árvore que guarda as colunas

	//O autômato. Esses nós são também as folhas da árvore que guarda as colunas
	vector<UkkonenAutomatonNode> automaton;
	
	const char* pattern; //O padrão
	size_t m; //O tamanho do padrão

	size_t n; //O tamanho do texto apresentado até agora

	int error;

	int initialAutomatonState; //O estado inicial do autômato
	//O estado atual do autômato, alcançado com o texto apresentado até agora
	int currentAutomatonState; 

	/*
	 * Recupera o nó do autômato que representa a coluna 'col'.
	 * Caso essa coluna não tenha representação, ela é 
	 * automaticamente inserida. Nesse caso, a flag 'wasCreated'
	 * é definida como 'true'
	 */
	int getColumnFromTree(int* col, bool* wasCreated);

	void calculateNextColumn(int* col, int ch, int* nextCol);
	int buildFDA();

};

//Pool de 'colunas' para o algoritmo de construção do FDA
struct MemPool {
	#define MAX_SIZE 1000000
	int* pool[MAX_SIZE];
	int count;
	int m;

	MemPool(int m): count(0), m(m){}
	~MemPool(){
		for(int i = 0; i < count; ++i)
			delete pool[i];
	}
	void giveBack(int* mem) {
		if(count == MAX_SIZE) delete mem;
		else
			pool[count++] = mem;
	}

	int* borrow() {
		if(!count) return new int[m+1];
		return pool[--count];
	}
	#undef MAX_SIZE
};

#endif