#ifndef GLOBAL
#define GLOBAL

#define MAX_ALPHABET_SIZE 256
#define ALPHABET_CONVERT(a) ((a)+128)
#define ALPHABET_INDEX(ch) Alphabet::mapping[ALPHABET_CONVERT(ch)]
#define ALPHABET_SIZE Alphabet::size

#define MIN(a,b) ((a)<(b)?(a):(b))
#define MAX(a,b) ((a)>(b)?(a):(b))

#include <unistd.h>
#include <cstring>
#include <cstdio>

/*
 * Utilizamos essa definição para que possamos modificar com facilidade
 * o tamanho da palavra utilizada pelo MyBinaryInteger e pelo ShiftOr2.
 */
typedef unsigned long long ull;
#define ONE 1ULL

#define RED_COLOR "\033[31m"
#define YELLOW_COLOR "\033[33m"
#define RESET_COLOR "\033[0m"

/*
 * A impressão de cores só deve ser disponibilizada caso o programa
 * esteja imprimindo diretamente no terminal.
 */
#define COLORED_PRINTING_IS_ENABLED (isColoredPrintingEnabled())
extern int isColoredPrintingEnabled();

/*
 * O alfabeto consiste das letras presentes no padrão mais o caracter
 * fim-de-cadeia (ASCII 0) que representa todo caracter não presente
 * no padrão. 
 * É feito um mapeamento de todo caracter ASCII para os caracteres do 
 * alfabeto. Dessa forma reduz-se o espaço/tempo de execução dos
 * algoritmos que fazem uso direto do tamanho do alfabeto (e.g, Aho, Ukkonen).
 */
namespace Alphabet {
	extern int mapping[MAX_ALPHABET_SIZE];
	extern int size;

	//Inicializa o alfabeto com um padrão
	extern void setup(const char* pattern, size_t m);

	//Constrói o alfabeto incrementalmente, apresentando possivelmente vários padrões.
	//Utilizado pelo Aho Corasick.
	extern void incrementalSetup(const char* pattern, size_t m);
};

#endif