#include "Aho.h"

Aho::Aho(bool countOnly, const vector<const char*>& patterns): countOnly(countOnly) {
	currentNode = root = 0;
	automaton.push_back(AutomatonNode());
	automaton.back().setup();

	n = 0;
	numberOfMatchings = 0;
	biggestPattern = 0;

	for(int i = 0; i < patterns.size(); ++i) 
		addPattern(patterns[i]);

	buildFailureLinks();
}

int Aho::update(char* chunk, size_t chunkSize) {
	int initialPos = (n==0) ? 0 : biggestPattern - 1;

	for(int c = initialPos; c < chunkSize; ++c) {
		while(currentNode != root && automaton[currentNode].nextFn[ALPHABET_INDEX(chunk[c])] == -1)
			currentNode = automaton[currentNode].failFn;

		if(automaton[currentNode].nextFn[ALPHABET_INDEX(chunk[c])] != -1) 
			currentNode = automaton[currentNode].nextFn[ALPHABET_INDEX(chunk[c])];

		if(automaton[currentNode].listOfPatterns->size) 
			registerAMatching(chunk, c, automaton[currentNode].listOfPatterns);

		++n;
	}

	return biggestPattern - 1;
}

void Aho::registerAMatching(char* chunk, int matchingPos, AhoList* listOfPatterns) {
	numberOfMatchings += listOfPatterns->size;

	if(!countOnly) {
		for(AhoNode* cur = listOfPatterns->first; cur != NULL; cur = cur -> next){
			int m = cur->size;
			int initialPos = matchingPos - m + 1, endingPos = matchingPos;
			for(int i = 0; i < 10 && initialPos > 0 && chunk[initialPos-1] != '\n'; ++i, --initialPos);
			for(int i = 0; i < 10 && chunk[endingPos+1] != 0 && chunk[endingPos+1] != '\n'; ++i, ++endingPos);
			
			//Imprime os caracteres antes da acorrência
			if(initialPos <= matchingPos - m) {
				char tmp = chunk[matchingPos - m + 1];
				chunk[matchingPos - m + 1] = 0;
				printf("%s", chunk + initialPos);
				chunk[matchingPos - m + 1] = tmp;
			}

			//Imprime a ocorrência (possivelmente colorida)
			if(COLORED_PRINTING_IS_ENABLED) printf("%s", RED_COLOR);
			char tmp = chunk[matchingPos + 1];
			chunk[matchingPos+1] = 0;
			printf("%s", chunk + matchingPos - m + 1);
			chunk[matchingPos+1] = tmp;
			if(COLORED_PRINTING_IS_ENABLED) printf("%s", RESET_COLOR);
			
			//Imprime o caracteres depois da ocorrência
			if(endingPos > matchingPos) {
				char tmp = chunk[endingPos+1];
				chunk[endingPos+1] = 0;
				printf("%s", chunk + matchingPos + 1);
				chunk[endingPos+1] = tmp;
			}

			//Imprime uma quebra de linha
			puts("");
		}
	}
}

ull Aho::getNumberOfMatchings() {
	return numberOfMatchings;
}

void Aho::addPattern(const char* pattern) {
	int cur = root;
	int i;
	for(i = 0; pattern[i]; ++i) {
		if(automaton[cur].nextFn[ALPHABET_INDEX(pattern[i])] == -1) {
			automaton[cur].nextFn[ALPHABET_INDEX(pattern[i])] = automaton.size();
			automaton.push_back(AutomatonNode());
			automaton.back().setup();
		}
		cur = automaton[cur].nextFn[ALPHABET_INDEX(pattern[i])];
	}

	biggestPattern = MAX(biggestPattern, i);
	automaton[cur].listOfPatterns->add(pattern, i);
}

void Aho::buildFailureLinks() {
	int* queue = new int[automaton.size()];
	int ini = 0, fim = 0;

	for(int i = 0; i < ALPHABET_SIZE; ++i) {
		int next = automaton[root].nextFn[i];
		if(next != -1) {
			automaton[next].failFn = root;
			queue[fim++] = next;
		}
	}

	while(ini < fim) {
		int cur = queue[ini++];

		for(int i = 0; i < ALPHABET_SIZE; ++i) {
			int next = automaton[cur].nextFn[i];
			if(next != -1) {
				queue[fim++] = next;
				int fail = automaton[cur].failFn;

				while(fail != root && automaton[fail].nextFn[i] == -1)
					fail = automaton[fail].failFn;

				if(automaton[fail].nextFn[i] != -1)
					fail = automaton[fail].nextFn[i];

				automaton[next].failFn = fail;
				automaton[next].listOfPatterns -> concatenate(automaton[fail].listOfPatterns);
			}
		}
	}

	delete [] queue;
}

