#include "BoyerMoore.h"

BoyerMoore::BoyerMoore(bool countOnly, const char* pattern, size_t m): ExactMatcher(countOnly){
	this->pattern = pattern;
	this->m = m;

	n = 0;
	C = new int[ALPHABET_SIZE];
	S = new int[m];
	setupGoodSuffix();

	setupBadChar();

	charsFromPreviousChunk = 0;
}

void BoyerMoore::setupGoodSuffix() {
	int* borders = new int[m];
	buildReversedBorder(borders);
		
	//Algoritmo final, derivado dos lemas 1-3
	for(int i = 0; i < m; ++i)
		S[i] = m - borders[m-1];
	
	//t é um tamanho. Por isso, precisamos fazer t-1 para consultar 
	//o array borders
	for(int t = 1; t < m; ++t) 
		S[m-1-borders[t-1]] = MIN(S[m-1-borders[t-1]], t - borders[t-1]);

	delete borders;
}

void BoyerMoore::buildReversedBorder(int* borders) {
	int k = borders[0] = -1;

	for(int i = 1; i < m; ++i) {
		while(k != -1 && pattern[m-1-i] != pattern[m-1-(k+1)])
			k = borders[k];
		if(pattern[m-1-i] == pattern[m-1-(k+1)])
			++k;
		borders[i] = k;
	}

	//cálculo do tamanho das bordas a partir da função de falha
	for(int i = 0; i < m; ++i) 
		++borders[i];
}

void BoyerMoore::setupBadChar() {
	for(int i = 0; i < ALPHABET_SIZE; ++i)
		C[i] = -1;

	for(int i = 0; i < m; ++i)
		C[ALPHABET_INDEX(pattern[i])] = i;
}

int BoyerMoore::update(char* chunk, size_t chunkSize) {
	int endOfWindow = m-1;

	while(endOfWindow < chunkSize) {
		int j = m - 1, i = endOfWindow;
		while(j >= 0 && chunk[i] == pattern[j]) {
			--j, --i;
		}

		if(j < 0){
			registerAMatching(chunk, endOfWindow, m);
			endOfWindow += S[0];
		}else
			endOfWindow += MAX(j - C[ALPHABET_INDEX(chunk[i])], S[j]);
	}

	n += chunkSize - charsFromPreviousChunk;

	charsFromPreviousChunk = m - (endOfWindow - chunkSize + 1);
	return charsFromPreviousChunk;
}

