#include "Kmp.h"

Kmp::Kmp(bool countOnly, const char* pattern, size_t m) : ExactMatcher(countOnly) { 
	this->m = m;
	this->pattern = pattern;

	fail = new int[m];
	p = -1;

	buildFailureFunction();
}

void Kmp::buildFailureFunction() {
	int k = fail[0] = -1; //k  é sempre fail[i-1]

	//contra caso para borda estrita: DDDE
	for(int i = 1; i < m; ++i) {
		while(k != -1 && pattern[k+1] != pattern[i])
			k = fail[k];

		if(pattern[k+1] == pattern[i])
			++k;

		fail[i] = k;
	}
}

int Kmp::update(char* chunk, size_t chunkSize) {
	int initialPosition = (n == 0) ? 0 : (m-1);

	for(int c = initialPosition; c < chunkSize; ++c) {
		++n; //aumenta o tamanho atual do texto
		char nextCharFromText = chunk[c];

		while(p != -1 && pattern[p+1] != nextCharFromText)
			p = fail[p];

		if(pattern[p+1] == nextCharFromText)
			++p;

		if(p == m-1) {//uma ocorrencia foi encontrada!
			registerAMatching(chunk, c, m);
			p = fail[p];
		}
	}

	return m-1;
}
